# air courier (air_courier.mod)

## airports
set Airports;
param Dist{Airports, Airports} >= 0, default Uniform(100,2000);
## aircrafts
set AircraftTypes;
# max volume per aircraft type
param AV{AircraftTypes} >= 0;
# max weight per aircraft type
param AW{AircraftTypes} >= 0;
# cost per mile per aircraft type
param ACpM{AircraftTypes} >= 0;
# number of flights on the time horizon
param DaysMax := 7;
set Days := 1..DaysMax;

## flight segment network
set Segments within {Airports, Airports};
param Aircraft{Segments} symbolic;
param VolumeCap{(u,v) in Segments} >= 0, default DaysMax*AV[Aircraft[u,v]];
param WeightCap{(u,v) in Segments} >= 0, default DaysMax*AW[Aircraft[u,v]];
param ArcCost{(u,v) in Segments} default ACpM[Aircraft[u,v]]*Dist[u,v];

## fine-grained demands
param Dmax;
set Demand;
param Volume{Demand} >= 0;
param Weight{Demand} >= 0;
param Orig{Demand} symbolic;
param Dest{Demand} symbolic;
# coarse demand
set D within {Airports,Airports};
param dV{D} >= 0, default 0;
param dW{D} >= 0, default 0;

## decision variables

# volume (unsplittable) flow
var V{Segments,D} binary;
# weight (unsplittable) flow
var W{Segments,D} binary;
# whether a flight leaves empty
var E{Segments,Days} binary;

## objective function
minimize cost: sum{(h,k) in D, (u,v) in Segments} ArcCost[u,v]*W[u,v,h,k];

## constraints

# volume multiflow
subject to volumeFlow{(h,k) in D, v in Airports}:
  sum{w in Airports: (v,w) in Segments} V[v,w,h,k] -
  sum{u in Airports: (u,v) in Segments} V[u,v,h,k] =
    if (v == h) then 1 else if (v == k) then -1 else 0;

subject to volumeCapacity{(u,v) in Segments}:
  sum{(h,k) in D} dV[h,k]*V[u,v,h,k] <= VolumeCap[u,v];

# weight multiflow
subject to weightFlow{(h,k) in D, v in Airports}:
  sum{w in Airports: (v,w) in Segments} W[v,w,h,k] -
  sum{u in Airports: (u,v) in Segments} W[u,v,h,k] =
    if (v == h) then 1 else if (v == k) then -1 else 0;

subject to weightCapacity{(u,v) in Segments}:
  sum{(h,k) in D} dW[h,k]*W[u,v,h,k] <= WeightCap[u,v];

# consistency
subject to consistent{(h,k) in D, (u,v) in Segments}:
  V[u,v,h,k] = W[u,v,h,k];

# company policy on non-empty flights (at least half full volume-wise)
subject to companypolicy1{(u,v) in Segments}:
  sum{(h,k) in D} dV[h,k]*V[u,v,h,k] >=
    (0.5*VolumeCap[u,v]/DaysMax)*sum{t in Days} E[u,v,t];
subject to companypolicy2{(u,v) in Segments}:
  sum{(h,k) in D} dV[h,k]*V[u,v,h,k] <=
    (VolumeCap[u,v]/DaysMax)*sum{t in Days} E[u,v,t];
